//开源许可证声明
/*
 * Copyright (c) 2010-2011, The MiCode Open Source Community (www.micode.net)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package net.micode.notes.gtask.data;

import android.database.Cursor;
import android.util.Log;

import net.micode.notes.data.Notes;
import net.micode.notes.data.Notes.NoteColumns;
import net.micode.notes.gtask.exception.ActionFailureException;
import net.micode.notes.tool.GTaskStringUtils;

import org.json.JSONException;
import org.json.JSONObject;

import java.util.ArrayList;

//这段代码定义了一个名为TaskList的Java类，它继承自Node类。
//这个类主要用于管理任务列表，包括创建、更新、设置内容、同步动作、添加、移除、移动子任务等功能。
public class TaskList extends Node {
    // 定义一个常量，用于打印日志
    private static final String TAG = TaskList.class.getSimpleName();

    // 定义一个变量，用于存储任务列表的索引
    private int mIndex;

    // 定义一个变量，用于存储任务列表的子任务
    private ArrayList<Task> mChildren;

    // 构造函数，初始化任务列表的子任务和索引
    public TaskList() {
        // 调用父类构造函数
        super();
        // 初始化mChildren为ArrayList<Task>类型
        mChildren = new ArrayList<Task>();
        // 初始化mIndex为1
        mIndex = 1;
    }

    // 获取创建任务的JSON对象
    public JSONObject getCreateAction(int actionId) {
        // 创建一个JSONObject对象
        JSONObject js = new JSONObject();

        try {
            // action_type
            // 将action_type设置为create
            js.put(GTaskStringUtils.GTASK_JSON_ACTION_TYPE,
                    GTaskStringUtils.GTASK_JSON_ACTION_TYPE_CREATE);

            // action_id
            // 将action_id设置为传入的参数
            js.put(GTaskStringUtils.GTASK_JSON_ACTION_ID, actionId);

            // index
            // 将index设置为成员变量mIndex的值
            js.put(GTaskStringUtils.GTASK_JSON_INDEX, mIndex);

            // entity_delta
            // 创建一个JSONObject对象entity
            JSONObject entity = new JSONObject();
            // 将entity的name设置为getName()的返回值
            entity.put(GTaskStringUtils.GTASK_JSON_NAME, getName());
            // 将entity的creator_id设置为null
            entity.put(GTaskStringUtils.GTASK_JSON_CREATOR_ID, "null");
            // 将entity的entity_type设置为group
            entity.put(GTaskStringUtils.GTASK_JSON_ENTITY_TYPE,
                    GTaskStringUtils.GTASK_JSON_TYPE_GROUP);
            // 将entity设置为entity_delta的值
            js.put(GTaskStringUtils.GTASK_JSON_ENTITY_DELTA, entity);

        } catch (JSONException e) {
            // 打印错误日志
            Log.e(TAG, e.toString());
            e.printStackTrace();
            // 抛出ActionFailureException异常
            throw new ActionFailureException("fail to generate tasklist-create jsonobject");
        }

        // 返回JSONObject对象
        return js;
    }

    // 获取更新任务的JSON对象
    public JSONObject getUpdateAction(int actionId) {
        // 创建一个JSONObject对象，用于存储更新任务列表的相关信息
        JSONObject js = new JSONObject();

        try {
            // action_type设置操作类型为更新任务列表
            js.put(GTaskStringUtils.GTASK_JSON_ACTION_TYPE,
                    GTaskStringUtils.GTASK_JSON_ACTION_TYPE_UPDATE);

            // action_id设置本次操作的唯一标识符
            js.put(GTaskStringUtils.GTASK_JSON_ACTION_ID, actionId);

            // id设置要更新的任务列表的全局唯一标识符
            js.put(GTaskStringUtils.GTASK_JSON_ID, getGid());

            // entity_delta创建一个新的 JSON 对象，用于存储任务列表实体的变更信息
            JSONObject entity = new JSONObject();
            // name: 设置任务列表的名称
            entity.put(GTaskStringUtils.GTASK_JSON_NAME, getName());
            // deleted: 设置任务列表的删除状态  
            entity.put(GTaskStringUtils.GTASK_JSON_DELETED, getDeleted());
            // updated: 设置任务列表的更新时间
            js.put(GTaskStringUtils.GTASK_JSON_ENTITY_DELTA, entity);

        } catch (JSONException e) {
            // 记录 JSON 解析异常的错误信息
            Log.e(TAG, e.toString());
            // 打印异常堆栈信息
            e.printStackTrace();
            // 抛出一个自定义的 ActionFailureException 异常，用于表示操作失败
            throw new ActionFailureException("fail to generate tasklist-update jsonobject");
        }

        return js;
    }

    // 根据远程JSON设置任务列表的内容
    public void setContentByRemoteJSON(JSONObject js) {
        // 根据远程JSON设置内容
        if (js != null) {
            try {
                // id
                if (js.has(GTaskStringUtils.GTASK_JSON_ID)) {
                    // 设置id
                    setGid(js.getString(GTaskStringUtils.GTASK_JSON_ID));
                }

                // last_modified
                if (js.has(GTaskStringUtils.GTASK_JSON_LAST_MODIFIED)) {
                    // 设置最后修改时间
                    setLastModified(js.getLong(GTaskStringUtils.GTASK_JSON_LAST_MODIFIED));
                }

                // name
                if (js.has(GTaskStringUtils.GTASK_JSON_NAME)) {
                    // 设置名称
                    setName(js.getString(GTaskStringUtils.GTASK_JSON_NAME));
                }

            } catch (JSONException e) {
                Log.e(TAG, e.toString());
                e.printStackTrace();
                // 抛出异常
                throw new ActionFailureException("fail to get tasklist content from jsonobject");
            }
        }
    }

    // 根据本地JSON设置任务列表的内容
    public void setContentByLocalJSON(JSONObject js) {
        // 如果JSON为空或者没有META_HEAD_NOTE字段，则打印警告日志
        if (js == null || !js.has(GTaskStringUtils.META_HEAD_NOTE)) {
            Log.w(TAG, "setContentByLocalJSON: nothing is avaiable");
        }

        try {
            // 获取META_HEAD_NOTE字段对应的JSONObject
            JSONObject folder = js.getJSONObject(GTaskStringUtils.META_HEAD_NOTE);

            // 如果类型为文件夹
            if (folder.getInt(NoteColumns.TYPE) == Notes.TYPE_FOLDER) {
                // 获取文件夹名称
                String name = folder.getString(NoteColumns.SNIPPET);
                // 设置名称
                setName(GTaskStringUtils.MIUI_FOLDER_PREFFIX + name);
            // 如果类型为系统文件夹
            } else if (folder.getInt(NoteColumns.TYPE) == Notes.TYPE_SYSTEM) {
                // 如果是根文件夹
                if (folder.getLong(NoteColumns.ID) == Notes.ID_ROOT_FOLDER)
                    // 设置名称
                    setName(GTaskStringUtils.MIUI_FOLDER_PREFFIX + GTaskStringUtils.FOLDER_DEFAULT);
                // 如果是通话记录文件夹
                else if (folder.getLong(NoteColumns.ID) == Notes.ID_CALL_RECORD_FOLDER)
                    // 设置名称
                    setName(GTaskStringUtils.MIUI_FOLDER_PREFFIX
                            + GTaskStringUtils.FOLDER_CALL_NOTE);
                // 否则打印错误日志
                else
                    Log.e(TAG, "invalid system folder");
            // 否则打印错误日志
            } else {
                Log.e(TAG, "error type");
            }
        // 捕获JSON异常
        } catch (JSONException e) {
            Log.e(TAG, e.toString());
            e.printStackTrace();
        }
    }

    // 获取本地JSON对象
    // 从内容中获取本地JSON
    public JSONObject getLocalJSONFromContent() {
        try {
            // 创建一个JSONObject对象
            JSONObject js = new JSONObject();
            // 创建一个JSONObject对象，用于存储文件夹信息
            JSONObject folder = new JSONObject();

            // 获取文件夹名称
            String folderName = getName();
            // 如果文件夹名称以MIUI_FOLDER_PREFFIX开头，则去掉该前缀
            if (getName().startsWith(GTaskStringUtils.MIUI_FOLDER_PREFFIX))
                folderName = folderName.substring(GTaskStringUtils.MIUI_FOLDER_PREFFIX.length(),
                        folderName.length());
            // 将文件夹名称放入folder对象中
            folder.put(NoteColumns.SNIPPET, folderName);
            // 如果文件夹名称是默认文件夹或通话记录文件夹，则将类型设置为系统类型
            if (folderName.equals(GTaskStringUtils.FOLDER_DEFAULT)
                    || folderName.equals(GTaskStringUtils.FOLDER_CALL_NOTE))
                folder.put(NoteColumns.TYPE, Notes.TYPE_SYSTEM);
            // 否则，将类型设置为文件夹类型
            else
                folder.put(NoteColumns.TYPE, Notes.TYPE_FOLDER);

            // 将folder对象放入js对象中
            js.put(GTaskStringUtils.META_HEAD_NOTE, folder);

            // 返回js对象
            return js;
        } catch (JSONException e) {
            // 打印错误日志
            Log.e(TAG, e.toString());
            e.printStackTrace();
            // 返回null
            return null;
        }
    }

    // 根据传入的 Cursor 对象判断任务列表的同步动作。
    //该方法会对比本地数据和远程数据的修改状态，从而决定采取何种同步操作。
    public int getSyncAction(Cursor c) {
        try {
            // 获取本地修改列的值，判断本地数据是否有更新
            if (c.getInt(SqlNote.LOCAL_MODIFIED_COLUMN) == 0) {
                // there is no local update本地没有更新
                // 比较本地同步 ID 列的值和远程任务列表的最后修改时间
                if (c.getLong(SqlNote.SYNC_ID_COLUMN) == getLastModified()) {
                    // no update both side 本地和远程都没有更新，无需同步操作
                    return SYNC_ACTION_NONE;
                } else {
                    // apply remote to local  远程有更新，本地无更新，需要将远程的更新应用到本地
                    return SYNC_ACTION_UPDATE_LOCAL;
                }
            } else {
                // validate gtask id本地有更新，验证本地存储的 Google 任务 ID 和当前任务列表的全局唯一标识符是否一致
                if (!c.getString(SqlNote.GTASK_ID_COLUMN).equals(getGid())) {
                    // 两者不一致，记录错误日志
                    Log.e(TAG, "gtask id doesn't match");
                    // 返回错误同步动作
                    return SYNC_ACTION_ERROR;
                }
                // 验证本地同步 ID 和远程任务列表的最后修改时间是否一致
                if (c.getLong(SqlNote.SYNC_ID_COLUMN) == getLastModified()) {
                    // local modification only 本地有修改，远程无更新，需要将本地的修改同步到远程
                    return SYNC_ACTION_UPDATE_REMOTE;
                } else {
                    // for folder conflicts, just apply local modification
                    // 对于文件夹冲突，直接将本地修改同步到远程
                    return SYNC_ACTION_UPDATE_REMOTE;
                }
            }
        } catch (Exception e) {
             // 捕获异常，记录错误日志
            Log.e(TAG, e.toString());
            // 打印异常堆栈信息
            e.printStackTrace();
        }
         // 发生异常或其他未知情况，返回错误同步动作
        return SYNC_ACTION_ERROR;
    }

    // 获取子任务数量
    public int getChildTaskCount() {
        // 返回子任务列表的大小
        return mChildren.size();
    }

    // 添加子任务
    public boolean addChildTask(Task task) {
        boolean ret = false;
        // 判断任务是否为空且不包含在子任务列表中
        if (task != null && !mChildren.contains(task)) {
            // 添加子任务
            ret = mChildren.add(task);
            // 如果添加成功
            if (ret) {
                // need to set prior sibling and parent
                task.setPriorSibling(mChildren.isEmpty() ? null : mChildren
                        .get(mChildren.size() - 1));
                task.setParent(this);
            }
        }
        return ret;
    }

    // 在指定位置添加子任务
    //如果指定的索引无效或者任务已经存在于列表中，操作将失败。
    //成功添加任务后，会更新相关任务的前一个兄弟任务引用。
    public boolean addChildTask(Task task, int index) {
        // 如果索引小于0或者大于子任务列表的大小，则返回false
        if (index < 0 || index > mChildren.size()) {
            Log.e(TAG, "add child task: invalid index");
            return false;
        }

        // 获取子任务在子任务列表中的位置
        int pos = mChildren.indexOf(task);
        // 如果子任务不为空且在子任务列表中不存在，则添加到指定位置
        if (task != null && pos == -1) {
            // 将任务添加到指定的索引位置
            mChildren.add(index, task);

            // update the task list
             // 定义两个任务对象，分别用于存储插入位置的前一个任务和后一个任务
            Task preTask = null;
             // 如果插入位置不是列表的第一个位置，获取插入位置的前一个任务
            if (index != 0)
            if (index != 0)
                preTask = mChildren.get(index - 1);
            // 如果插入位置不是列表的最后一个位置，获取插入位置的后一个任务
            if (index != mChildren.size() - 1)
                afterTask = mChildren.get(index + 1);
            // 设置新插入任务的前一个兄弟任务
            task.setPriorSibling(preTask);
            // 如果插入位置后面有任务，更新该任务的前一个兄弟任务为新插入的任务
            if (afterTask != null)
                afterTask.setPriorSibling(task);
        }

        return true;
    }

    // 移除子任务
    public boolean removeChildTask(Task task) {
        boolean ret = false;
        // 获取子任务在子任务列表中的索引
        int index = mChildren.indexOf(task);
        if (index != -1) {
            // 移除子任务
            ret = mChildren.remove(task);

            if (ret) {
                // reset prior sibling and parent
                task.setPriorSibling(null);
                task.setParent(null);

                // update the task list
                if (index != mChildren.size()) {
                    mChildren.get(index).setPriorSibling(
                            index == 0 ? null : mChildren.get(index - 1));
                }
            }
        }
        return ret;
    }

    // 移动子任务
    public boolean moveChildTask(Task task, int index) {

        // 如果索引小于0或者大于等于子任务列表的大小，则返回false
        if (index < 0 || index >= mChildren.size()) {
            Log.e(TAG, "move child task: invalid index");
            return false;
        }

        // 获取子任务在子任务列表中的位置
        int pos = mChildren.indexOf(task);
        // 如果子任务不在子任务列表中，则返回false
        if (pos == -1) {
            Log.e(TAG, "move child task: the task should in the list");
            return false;
        }

        // 如果子任务的位置和索引相同，则返回true
        if (pos == index)
            return true;
        // 否则，先移除子任务，再添加子任务到指定位置
        return (removeChildTask(task) && addChildTask(task, index));
    }

    // 根据GID查找子任务
    public Task findChildTaskByGid(String gid) {
        // 遍历子任务列表
        for (int i = 0; i < mChildren.size(); i++) {
            // 获取子任务
            Task t = mChildren.get(i);
            // 如果子任务的gid与传入的gid相等，则返回该子任务
            if (t.getGid().equals(gid)) {
                return t;
            }
        }
        // 如果没有找到相等的子任务，则返回null
        return null;
    }

    // 获取子任务在子任务列表中的索引
    public int getChildTaskIndex(Task task) {
        // 返回子任务在子任务列表中的索引
        return mChildren.indexOf(task);
    }

    // 根据索引获取子任务
    public Task getChildTaskByIndex(int index) {
        // 如果索引小于0或者大于等于子任务列表的大小，则输出错误信息并返回null
        if (index < 0 || index >= mChildren.size()) {
            Log.e(TAG, "getTaskByIndex: invalid index");
            return null;
        }
        // 返回子任务列表中指定索引的任务
        return mChildren.get(index);
    }

    // 根据GID获取子任务
    public Task getChilTaskByGid(String gid) {
        // 遍历子任务列表
        for (Task task : mChildren) {
            // 如果子任务的gid与传入的gid相等，则返回该子任务
            if (task.getGid().equals(gid))
                return task;
        }
        // 如果没有找到匹配的子任务，则返回null
        return null;
    }

    // 获取子任务列表
    public ArrayList<Task> getChildTaskList() {
        // 返回子任务列表
        return this.mChildren;
    }

    // 设置索引
    public void setIndex(int index) {
        // 将传入的索引赋值给成员变量mIndex
        this.mIndex = index;
    }

    // 获取索引
    public int getIndex() {
        // 返回索引
        return this.mIndex;
    }
}
