//开源许可证声明
/*
 * Copyright (c) 2010-2011, The MiCode Open Source Community (www.micode.net)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package net.micode.notes.model;
import android.content.ContentProviderOperation;
import android.content.ContentProviderResult;
import android.content.ContentUris;
import android.content.ContentValues;
import android.content.Context;
import android.content.OperationApplicationException;
import android.net.Uri;
import android.os.RemoteException;
import android.util.Log;

import net.micode.notes.data.Notes;
import net.micode.notes.data.Notes.CallNote;
import net.micode.notes.data.Notes.DataColumns;
import net.micode.notes.data.Notes.NoteColumns;
import net.micode.notes.data.Notes.TextNote;

import java.util.ArrayList;


//处理笔记（note）的数据和同步操作。
public class Note {
    // 定义一个ContentValues对象，用于存储note的差值
    private ContentValues mNoteDiffValues;
    // 定义一个NoteData对象，用于存储note的数据
    private NoteData mNoteData;
    // 定义一个TAG常量，用于日志输出
    private static final String TAG = "Note";
    /**
     * Create a new note id for adding a new note to databases
     */
    //同步方法，用于在数据库中创建一个新的笔记，并返回新笔记的ID。
    //它首先创建一个ContentValues对象，设置创建时间和修改时间，以及笔记类型和父文件夹ID，然后插入数据库并获取新笔记的ID。
    public static synchronized long getNewNoteId(Context context, long folderId) {
        // Create a new note in the database  在数据库中创建一个新的笔记
        ContentValues values = new ContentValues();
        // 获取当前系统时间作为笔记的创建时间
        long createdTime = System.currentTimeMillis();
        // 将创建时间插入到 ContentValues 中
        values.put(NoteColumns.CREATED_DATE, createdTime);
         // 将修改时间设置为与创建时间相同，并插入到 ContentValues 中
        values.put(NoteColumns.MODIFIED_DATE, createdTime);
        // 设置笔记类型为普通笔记，并插入到 ContentValues 中
        values.put(NoteColumns.TYPE, Notes.TYPE_NOTE);
        // 标记笔记已被本地修改，并插入到 ContentValues 中
        values.put(NoteColumns.LOCAL_MODIFIED, 1);
        // 设置笔记的父文件夹ID，并插入到 ContentValues 中
        values.put(NoteColumns.PARENT_ID, folderId);
         // 调用 ContentResolver 的 insert 方法将笔记信息插入到数据库中，并获取返回的 Uri
        Uri uri = context.getContentResolver().insert(Notes.CONTENT_NOTE_URI, values);

        long noteId = 0;
        try {
            // Get the note id from the uri从返回的 Uri 中提取笔记的 ID
            noteId = Long.valueOf(uri.getPathSegments().get(1));
        } catch (NumberFormatException e) {
             // 若提取 ID 时发生格式错误，记录错误日志并将笔记 ID 设置为 0
            Log.e(TAG, "Get note id error :" + e.toString());
            noteId = 0;
        }
        if (noteId == -1) {
            // 若笔记 ID 为 -1，抛出异常提示笔记 ID 错误
            throw new IllegalStateException("Wrong note id:" + noteId);
        }
        return noteId;
    }

    // 构造函数
    public Note() {
        // 创建一个ContentValues对象，用于存储数据
        mNoteDiffValues = new ContentValues();
        // 创建一个NoteData对象，用于存储数据
        mNoteData = new NoteData();
    }

    // 设置笔记的值
    public void setNoteValue(String key, String value) {
        // 将key和value放入mNoteDiffValues中
        mNoteDiffValues.put(key, value);
        // 将LOCAL_MODIFIED字段设置为1
        mNoteDiffValues.put(NoteColumns.LOCAL_MODIFIED, 1);
        // 将MODIFIED_DATE字段设置为当前时间
        mNoteDiffValues.put(NoteColumns.MODIFIED_DATE, System.currentTimeMillis());
    }

    // 设置文本数据
    public void setTextData(String key, String value) {
        // 调用mNoteData对象的setTextData方法，传入key和value参数
        mNoteData.setTextData(key, value);
    }

    // 设置文本数据ID
    public void setTextDataId(long id) {
        // 调用mNoteData对象的setTextDataId方法，传入参数id
        mNoteData.setTextDataId(id);
    }

    // 获取文本数据ID
    public long getTextDataId() {
        // 返回NoteData对象中的文本数据ID
        return mNoteData.mTextDataId;
    }

    // 设置通话数据ID
    public void setCallDataId(long id) {
        // 调用mNoteData对象的setCallDataId方法，传入参数id
        mNoteData.setCallDataId(id);
    }

    // 设置通话数据
    public void setCallData(String key, String value) {
        // 调用mNoteData对象的setCallData方法，传入key和value参数
        mNoteData.setCallData(key, value);
    }

    // 判断笔记是否被本地修改
    public boolean isLocalModified() {
        // 如果笔记的差值列表不为空，或者笔记数据被本地修改，则返回true
        return mNoteDiffValues.size() > 0 || mNoteData.isLocalModified();
    }

    // 同步笔记，将笔记的修改同步到数据库中。
    // 同步笔记的方法，用于将本地的笔记数据同步到云端。
    public boolean syncNote(Context context, long noteId) {
        // 检查noteId是否小于等于0，如果是，则抛出异常
        if (noteId <= 0) {
            throw new IllegalArgumentException("Wrong note id:" + noteId);
        }

        // 如果数据没有修改，则直接返回true
        if (!isLocalModified()) {
            return true;
        }

        /**
         * In theory, once data changed, the note should be updated on {@link NoteColumns#LOCAL_MODIFIED} and
         * {@link NoteColumns#MODIFIED_DATE}. For data safety, though update note fails, we also update the
         * note data info
         */
        /**
         * 理论上，一旦数据发生变化，笔记的 {@link } 和
         * {@link } 字段应该被更新。为了数据安全，即使更新笔记失败，
         * 我们仍然会更新笔记的数据信息。
         */
        // 更新笔记的差值列表到数据库中，如果更新失败，则打印错误日志
        if (context.getContentResolver().update(
                ContentUris.withAppendedId(Notes.CONTENT_NOTE_URI, noteId), mNoteDiffValues, null,
                null) == 0) {
            //如果更新行数为 0，说明更新失败，记录错误日志
            Log.e(TAG, "Update note error, should not happen");
            // Do not return, fall through
        }
        // 清空笔记的差值列表
        mNoteDiffValues.clear();
        // 检查笔记数据是否有本地修改，并且尝试将笔记数据同步到数据库中
        // 如果笔记数据有修改且同步失败（pushIntoContentResolver 返回 null），则返回 false 表示同步失败
        if (mNoteData.isLocalModified()
                && (mNoteData.pushIntoContentResolver(context, noteId) == null)) {
            return false;
        }

        return true;
    }

    private class NoteData {
        // 文本数据ID
        private long mTextDataId;

        // 文本数据内容
        private ContentValues mTextDataValues;

        // 通话数据ID
        private long mCallDataId;

        // 通话数据内容
        private ContentValues mCallDataValues;

        // 日志标签
        private static final String TAG = "NoteData";

        // 构造函数，初始化NoteData对象
        public NoteData() {
            // 初始化mTextDataValues对象
            mTextDataValues = new ContentValues();
            // 初始化mCallDataValues对象
            mCallDataValues = new ContentValues();
            // 初始化mTextDataId为0
            mTextDataId = 0;
            // 初始化mCallDataId为0
            mCallDataId = 0;
        }

        // 判断文本数据和通话数据是否被修改
        boolean isLocalModified() {
            // 如果文本数据值列表的大小大于0，或者通话数据值列表的大小大于0，则返回true
            return mTextDataValues.size() > 0 || mCallDataValues.size() > 0;
        }

        // 设置文本数据ID
        void setTextDataId(long id) {
            // 如果ID小于等于0，抛出异常
            if(id <= 0) {
                throw new IllegalArgumentException("Text data id should larger than 0");
            }
            // 设置文本数据ID
            mTextDataId = id;
        }

        // 设置通话数据ID
        void setCallDataId(long id) {
            // 如果id小于等于0，抛出异常
            if (id <= 0) {
                throw new IllegalArgumentException("Call data id should larger than 0");
            }
            // 设置mCallDataId为id
            mCallDataId = id;
        }

        // 设置通话数据
        void setCallData(String key, String value) {
            // 将key和value存入mCallDataValues中
            mCallDataValues.put(key, value);
            // 将LOCAL_MODIFIED字段设置为1
            mNoteDiffValues.put(NoteColumns.LOCAL_MODIFIED, 1);
            // 将MODIFIED_DATE字段设置为当前时间
            mNoteDiffValues.put(NoteColumns.MODIFIED_DATE, System.currentTimeMillis());
        }

        // 设置文本数据
        void setTextData(String key, String value) {
            // 将key和value存入mTextDataValues中
            mTextDataValues.put(key, value);
            // 将LOCAL_MODIFIED字段设置为1
            mNoteDiffValues.put(NoteColumns.LOCAL_MODIFIED, 1);
            // 将MODIFIED_DATE字段设置为当前时间
            mNoteDiffValues.put(NoteColumns.MODIFIED_DATE, System.currentTimeMillis());
        }

        // 将数据推送到ContentResolver
        //根据数据的状态（是否已有 ID）决定是插入新数据还是更新现有数据。
        // 如果有多个操作需要执行，会使用批量操作来提高效率。
        Uri pushIntoContentResolver(Context context, long noteId) {
            /**
             * Check for safety 安全检查，确保传入的笔记 ID 是有效的。
             */
            if (noteId <= 0) {
                // 若笔记 ID 无效，抛出异常提示
                throw new IllegalArgumentException("Wrong note id:" + noteId);
            }
            //创建一个存储 ContentProvider 操作的列表，用于批量执行操作
            ArrayList<ContentProviderOperation> operationList = new ArrayList<ContentProviderOperation>();
            // 定义一个 ContentProvider 操作构建器，用于构建更新操作
            ContentProviderOperation.Builder builder = null;

            // 如果文本数据不为空，需要处理文本数据
            if(mTextDataValues.size() > 0) {
                // 将笔记 ID 关联到文本数据中
                mTextDataValues.put(DataColumns.NOTE_ID, noteId);
                // 如果文本数据ID为0，则插入新数据
                if (mTextDataId == 0) {
                    // 设置文本数据的 MIME 类型
                    mTextDataValues.put(DataColumns.MIME_TYPE, TextNote.CONTENT_ITEM_TYPE);
                    // 调用 ContentResolver 的 insert 方法插入新的文本数据，并获取返回的 Uri
                    Uri uri = context.getContentResolver().insert(Notes.CONTENT_DATA_URI,
                            mTextDataValues);
                    try {
                        // 从返回的 Uri 中提取新插入文本数据的 ID，并设置到 mTextDataId 中
                        setTextDataId(Long.valueOf(uri.getPathSegments().get(1)));
                    } catch (NumberFormatException e) {
                        // 若提取 ID 时发生格式错误，记录错误日志并清空文本数据
                        Log.e(TAG, "Insert new text data fail with noteId" + noteId);
                        mTextDataValues.clear();
                        return null;
                    }
                //  否则，说明文本数据已存在，需要更新现有数据
                } else {
                     // 创建一个更新操作的构建器，指定要更新的数据 Uri
                    builder = ContentProviderOperation.newUpdate(ContentUris.withAppendedId(
                            Notes.CONTENT_DATA_URI, mTextDataId));
                    // 设置要更新的数据值
                    builder.withValues(mTextDataValues);
                    // 将更新操作添加到操作列表中
                    operationList.add(builder.build());
                }
                // 清空文本数据值
                mTextDataValues.clear();
            }

            // 如果通话数据不为空
            if(mCallDataValues.size() > 0) {
                mCallDataValues.put(DataColumns.NOTE_ID, noteId);
                // 如果通话数据ID为0，则插入新数据
                if (mCallDataId == 0) {
                    // 设置数据类型为通话记录类型
                    mCallDataValues.put(DataColumns.MIME_TYPE, CallNote.CONTENT_ITEM_TYPE);
                    // 插入新数据并获取插入后的 Uri
                    Uri uri = context.getContentResolver().insert(Notes.CONTENT_DATA_URI,
                            mCallDataValues);
                    try {
                        // 从 Uri 中提取 ID
                        setCallDataId(Long.valueOf(uri.getPathSegments().get(1)));
                    } catch (NumberFormatException e) {
                        // 若提取 ID 时发生格式错误，记录错误日志并清空数据
                        Log.e(TAG, "Insert new call data fail with noteId" + noteId);
                        mCallDataValues.clear();
                        return null;
                    }
                // 否则更新数据
                } else {
                    // 创建一个更新操作的构建器，指定要更新的数据 Uri
                    builder = ContentProviderOperation.newUpdate(ContentUris.withAppendedId(
                            Notes.CONTENT_DATA_URI, mCallDataId));
                    // 设置要更新的数据值
                    builder.withValues(mCallDataValues);
                    // 将更新操作添加到操作列表中
                    operationList.add(builder.build());
                }
                // 清空数据值
                mCallDataValues.clear();
            }

            // 如果操作列表不为空，则执行批量操作
            if (operationList.size() > 0) {
                try {
                    // 执行批量操作
                    ContentProviderResult[] results = context.getContentResolver().applyBatch(
                            Notes.AUTHORITY, operationList);
                    // 返回更新后的数据 Uri
                    return (results == null || results.length == 0 || results[0] == null) ? null
                            : ContentUris.withAppendedId(Notes.CONTENT_NOTE_URI, noteId);
                } catch (RemoteException e) {
                    // 若远程操作异常，记录错误日志并返回 null
                    Log.e(TAG, String.format("%s: %s", e.toString(), e.getMessage()));
                    return null;
                } catch (OperationApplicationException e) {
                    // 若操作应用异常，记录错误日志并返回 null
                    Log.e(TAG, String.format("%s: %s", e.toString(), e.getMessage()));
                    return null;
                }
            }
            return null;
        }
    }
}
