//开源许可证声明
/*
 * Copyright (c) 2010-2011, The MiCode Open Source Community (www.micode.net)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package net.micode.notes.model;

import android.appwidget.AppWidgetManager;
import android.content.ContentUris;
import android.content.Context;
import android.database.Cursor;
import android.text.TextUtils;
import android.util.Log;

import net.micode.notes.data.Notes;
import net.micode.notes.data.Notes.CallNote;
import net.micode.notes.data.Notes.DataColumns;
import net.micode.notes.data.Notes.DataConstants;
import net.micode.notes.data.Notes.NoteColumns;
import net.micode.notes.data.Notes.TextNote;
import net.micode.notes.tool.ResourceParser.NoteBgResources;

//用于处理笔记（Note）的相关操作。
//这个类包含了笔记的各种属性，如内容、模式、警报日期、修改日期、背景颜色ID、小部件ID、小部件类型、文件夹ID等。
//它还定义了一些方法来创建新的笔记、加载现有的笔记、保存笔记、设置警报日期、标记笔记为已删除、设置背景颜色、设置清单模式、设置小部件类型、设置小部件ID、设置工作文本、转换为电话笔记等。
public class WorkingNote {
    
    // Note对象
    private Note mNote;
    
    // Note的ID
    private long mNoteId;
    
    // Note的内容
    private String mContent;
    
    // Note的模式
    private int mMode;

    // 提醒日期
    private long mAlertDate;

    // 修改日期
    private long mModifiedDate;

    // 背景颜色ID
    private int mBgColorId;

    // 小部件ID
    private int mWidgetId;

    // 小部件类型
    private int mWidgetType;

    // 文件夹ID
    private long mFolderId;

    // 上下文
    private Context mContext;

    // 日志标签
    private static final String TAG = "WorkingNote";

    // 是否已删除
    private boolean mIsDeleted;

    // Note设置状态监听器
    private NoteSettingChangedListener mNoteSettingStatusListener;

    // 数据投影
    public static final String[] DATA_PROJECTION = new String[] {
            DataColumns.ID,
            DataColumns.CONTENT,
            DataColumns.MIME_TYPE,
            DataColumns.DATA1,
            DataColumns.DATA2,
            DataColumns.DATA3,
            DataColumns.DATA4,
    };

    // Note投影
    public static final String[] NOTE_PROJECTION = new String[] {
            NoteColumns.PARENT_ID,
            NoteColumns.ALERTED_DATE,
            NoteColumns.BG_COLOR_ID,
            NoteColumns.WIDGET_ID,
            NoteColumns.WIDGET_TYPE,
            NoteColumns.MODIFIED_DATE
    };

    // 数据ID列
    private static final int DATA_ID_COLUMN = 0;

    // 数据内容列
    private static final int DATA_CONTENT_COLUMN = 1;

    // 数据MIME类型列
    private static final int DATA_MIME_TYPE_COLUMN = 2;

    // 数据模式列
    private static final int DATA_MODE_COLUMN = 3;

    // Note父ID列
    private static final int NOTE_PARENT_ID_COLUMN = 0;

    // Note提醒日期列
    private static final int NOTE_ALERTED_DATE_COLUMN = 1;

    // Note背景颜色ID列
    private static final int NOTE_BG_COLOR_ID_COLUMN = 2;

    // Note小部件ID列
    private static final int NOTE_WIDGET_ID_COLUMN = 3;

    // Note小部件类型列
    private static final int NOTE_WIDGET_TYPE_COLUMN = 4;

    // Note修改日期列
    private static final int NOTE_MODIFIED_DATE_COLUMN = 5;

    // New note construct
    // 构造函数，用于创建一个WorkingNote对象
    private WorkingNote(Context context, long folderId) {
        // 保存上下文
        mContext = context;
        // 初始化提醒日期为0
        mAlertDate = 0;
        // 初始化修改日期为当前时间
        mModifiedDate = System.currentTimeMillis();
        // 保存文件夹ID
        mFolderId = folderId;
        // 创建一个新的Note对象
        mNote = new Note();
        // 初始化NoteID为0
        mNoteId = 0;
        // 初始化删除状态为false
        mIsDeleted = false;
        // 初始化模式为0
        mMode = 0;
        // 初始化小部件类型为无效
        mWidgetType = Notes.TYPE_WIDGET_INVALIDE;
    }

    // Existing note construct
    // 构造函数，用于创建一个WorkingNote对象
    private WorkingNote(Context context, long noteId, long folderId) {
        // 保存上下文
        mContext = context;
        // 保存笔记ID
        mNoteId = noteId;
        // 保存文件夹ID
        mFolderId = folderId;
        // 初始化删除状态为false
        mIsDeleted = false;
        // 创建一个新的Note对象
        mNote = new Note();
        // 加载笔记
        loadNote();
    }

    // 加载笔记
    private void loadNote() {
        // 从ContentResolver中查询指定id的笔记
        Cursor cursor = mContext.getContentResolver().query(
                ContentUris.withAppendedId(Notes.CONTENT_NOTE_URI, mNoteId), NOTE_PROJECTION, null,
                null, null);

        // 如果查询结果不为空
        if (cursor != null) {
            // 如果查询结果中有数据
            if (cursor.moveToFirst()) {
                // 获取笔记的父文件夹id
                mFolderId = cursor.getLong(NOTE_PARENT_ID_COLUMN);
                // 获取笔记的背景颜色id
                mBgColorId = cursor.getInt(NOTE_BG_COLOR_ID_COLUMN);
                // 获取笔记的widget id
                mWidgetId = cursor.getInt(NOTE_WIDGET_ID_COLUMN);
                // 获取笔记的widget类型
                mWidgetType = cursor.getInt(NOTE_WIDGET_TYPE_COLUMN);
                // 获取笔记的提醒日期
                mAlertDate = cursor.getLong(NOTE_ALERTED_DATE_COLUMN);
                // 获取笔记的修改日期
                mModifiedDate = cursor.getLong(NOTE_MODIFIED_DATE_COLUMN);
            }
            // 关闭查询结果
            cursor.close();
        } else {
            // 如果查询结果为空，则打印错误日志并抛出异常
            Log.e(TAG, "No note with id:" + mNoteId);
            throw new IllegalArgumentException("Unable to find note with id " + mNoteId);
        }
        // 加载笔记数据
        loadNoteData();
    }

    // 加载笔记数据
    private void loadNoteData() {
        // 从ContentResolver中查询笔记数据
        Cursor cursor = mContext.getContentResolver().query(Notes.CONTENT_DATA_URI, DATA_PROJECTION,
                DataColumns.NOTE_ID + "=?", new String[] {
                    String.valueOf(mNoteId)
                }, null);

        // 如果查询结果不为空
        if (cursor != null) {
            // 如果查询结果中有数据
            if (cursor.moveToFirst()) {
                // 遍历查询结果
                do {
                    // 获取数据类型
                    String type = cursor.getString(DATA_MIME_TYPE_COLUMN);
                    // 如果数据类型为笔记
                    if (DataConstants.NOTE.equals(type)) {
                        // 获取笔记内容
                        mContent = cursor.getString(DATA_CONTENT_COLUMN);
                        // 获取笔记模式
                        mMode = cursor.getInt(DATA_MODE_COLUMN);
                        // 设置笔记数据ID
                        mNote.setTextDataId(cursor.getLong(DATA_ID_COLUMN));
                    // 如果数据类型为电话笔记
                    } else if (DataConstants.CALL_NOTE.equals(type)) {
                        // 设置电话笔记数据ID
                        mNote.setCallDataId(cursor.getLong(DATA_ID_COLUMN));
                    // 如果数据类型不正确
                    } else {
                        // 打印错误日志
                        Log.d(TAG, "Wrong note type with type:" + type);
                    }
                } while (cursor.moveToNext());
            }
            // 关闭查询结果
            cursor.close();
        // 如果查询结果为空
        } else {
            // 打印错误日志
            Log.e(TAG, "No data with id:" + mNoteId);
            // 抛出异常
            throw new IllegalArgumentException("Unable to find note's data with id " + mNoteId);
        }
    }

    // 创建一个空的WorkingNote对象
    public static WorkingNote createEmptyNote(Context context, long folderId, int widgetId,
            int widgetType, int defaultBgColorId) {
        // 创建一个新的WorkingNote对象，传入上下文、文件夹ID
        WorkingNote note = new WorkingNote(context, folderId);
        // 设置背景颜色ID
        note.setBgColorId(defaultBgColorId);
        // 设置小部件ID
        note.setWidgetId(widgetId);
        // 设置小部件类型
        note.setWidgetType(widgetType);
        // 返回创建的WorkingNote对象
        return note;
    }

    // 从上下文和id加载工作笔记
    public static WorkingNote load(Context context, long id) {
        // 创建一个新的工作笔记对象，传入上下文、id和0
        return new WorkingNote(context, id, 0);
    }
    // 保存笔记
    // 同步方法，用于保存笔记
    public synchronized boolean saveNote() {
        // 如果需要保存
        if (isWorthSaving()) {
            // 如果数据库中不存在
            if (!existInDatabase()) {
                // 获取新的笔记ID
                if ((mNoteId = Note.getNewNoteId(mContext, mFolderId)) == 0) {
                    // 如果获取失败，打印错误日志
                    Log.e(TAG, "Create new note fail with id:" + mNoteId);
                    return false;
                }
            }

            // 同步笔记
            mNote.syncNote(mContext, mNoteId);

            /**
             * Update widget content if there exist any widget of this note
             */
            if (mWidgetId != AppWidgetManager.INVALID_APPWIDGET_ID
                    && mWidgetType != Notes.TYPE_WIDGET_INVALIDE
                    && mNoteSettingStatusListener != null) {
                mNoteSettingStatusListener.onWidgetChanged();
            }
            return true;
        } else {
            return false;
        }
    }

    // 判断当前对象是否存在于数据库中
    public boolean existInDatabase() {
        // 如果mNoteId大于0，则表示当前对象存在于数据库中
        return mNoteId > 0;
    }

    // 判断是否值得保存
    private boolean isWorthSaving() {
        // 如果已经被删除，或者不存在于数据库中且内容为空，或者存在于数据库中但未修改
        if (mIsDeleted || (!existInDatabase() && TextUtils.isEmpty(mContent))
                || (existInDatabase() && !mNote.isLocalModified())) {
            // 不值得保存
            return false;
        } else {
            // 值得保存
            return true;
        }
    }

    // 设置NoteSettingChangedListener监听器
    public void setOnSettingStatusChangedListener(NoteSettingChangedListener l) {
        // 将传入的监听器赋值给mNoteSettingStatusListener
        mNoteSettingStatusListener = l;
    }

    // 设置提醒日期
    public void setAlertDate(long date, boolean set) {
        // 如果传入的日期与当前日期不同
        if (date != mAlertDate) {
            // 更新当前日期
            mAlertDate = date;
            // 将日期转换为字符串并设置到Note对象中
            mNote.setNoteValue(NoteColumns.ALERTED_DATE, String.valueOf(mAlertDate));
        }
        // 如果设置了监听器
        if (mNoteSettingStatusListener != null) {
            // 调用监听器的onClockAlertChanged方法，传入日期和设置状态
            mNoteSettingStatusListener.onClockAlertChanged(date, set);
        }
    }

    // 标记是否已删除
    public void markDeleted(boolean mark) {
        // 将mIsDeleted设置为mark
        mIsDeleted = mark;
        // 如果mWidgetId不为INVALID_APPWIDGET_ID，mWidgetType不为INVALIDE，并且mNoteSettingStatusListener不为null
        if (mWidgetId != AppWidgetManager.INVALID_APPWIDGET_ID
                && mWidgetType != Notes.TYPE_WIDGET_INVALIDE && mNoteSettingStatusListener != null) {
            // 调用mNoteSettingStatusListener的onWidgetChanged方法
                mNoteSettingStatusListener.onWidgetChanged();
        }
    }

    // 设置背景颜色ID
    public void setBgColorId(int id) {
        // 如果传入的ID与当前背景颜色ID不同
        if (id != mBgColorId) {
            // 更新当前背景颜色ID
            mBgColorId = id;
            // 如果有监听器，则调用监听器的onBackgroundColorChanged方法
            if (mNoteSettingStatusListener != null) {
                mNoteSettingStatusListener.onBackgroundColorChanged();
            }
            // 更新笔记的背景颜色ID
            mNote.setNoteValue(NoteColumns.BG_COLOR_ID, String.valueOf(id));
        }
    }

    // 设置检查列表模式
    public void setCheckListMode(int mode) {
        // 如果当前模式与传入的模式不同
        if (mMode != mode) {
            // 如果有监听器，则调用监听器的onCheckListModeChanged方法
            if (mNoteSettingStatusListener != null) {
                mNoteSettingStatusListener.onCheckListModeChanged(mMode, mode);
            }
            // 更新模式
            mMode = mode;
            // 将模式保存到文本数据中
            mNote.setTextData(TextNote.MODE, String.valueOf(mMode));
        }
    }

    // 设置小部件类型
    public void setWidgetType(int type) {
        // 如果传入的类型与当前类型不同
        if (type != mWidgetType) {
            // 更新当前类型
            mWidgetType = type;
            // 将当前类型保存到NoteColumns.WIDGET_TYPE字段中
            mNote.setNoteValue(NoteColumns.WIDGET_TYPE, String.valueOf(mWidgetType));
        }
    }

    // 设置小部件ID
    public void setWidgetId(int id) {
        // 如果传入的ID与当前的小部件ID不同
        if (id != mWidgetId) {
            // 将当前的小部件ID设置为传入的ID
            mWidgetId = id;
            // 将小部件ID保存到Note对象中
            mNote.setNoteValue(NoteColumns.WIDGET_ID, String.valueOf(mWidgetId));
        }
    }

    // 设置工作文本
    public void setWorkingText(String text) {
        // 如果工作文本和传入的文本不相等
        if (!TextUtils.equals(mContent, text)) {
            // 将传入的文本赋值给工作文本
            mContent = text;
            // 将工作文本设置到Note对象中
            mNote.setTextData(DataColumns.CONTENT, mContent);
        }
    }

    // 将电话号码和通话日期转换为CallNote
    public void convertToCallNote(String phoneNumber, long callDate) {
        // 将通话日期转换为字符串并设置到CallNote中
        mNote.setCallData(CallNote.CALL_DATE, String.valueOf(callDate));
        // 将电话号码设置到CallNote中
        mNote.setCallData(CallNote.PHONE_NUMBER, phoneNumber);
        // 设置CallNote的父ID为通话记录文件夹的ID
        mNote.setNoteValue(NoteColumns.PARENT_ID, String.valueOf(Notes.ID_CALL_RECORD_FOLDER));
    }

    // 判断是否有闹钟提醒
    public boolean hasClockAlert() {
        // 如果mAlertDate大于0，则返回true，否则返回false
        return (mAlertDate > 0 ? true : false);
    }

    // 获取内容
    public String getContent() {
        // 返回mContent
        return mContent;
    }

    // 获取提醒日期
    public long getAlertDate() {
        // 返回提醒日期
        return mAlertDate;
    }

    // 获取修改日期
    public long getModifiedDate() {
        // 返回修改日期
        return mModifiedDate;
    }

    // 获取背景颜色资源ID
    public int getBgColorResId() {
        // 根据背景颜色ID获取对应的背景颜色资源ID
        return NoteBgResources.getNoteBgResource(mBgColorId);
    }

    // 获取背景颜色ID
    public int getBgColorId() {
        // 返回背景颜色ID
        return mBgColorId;
    }

    // 获取标题背景资源ID
    public int getTitleBgResId() {
        // 根据背景颜色ID获取标题背景资源ID
        return NoteBgResources.getNoteTitleBgResource(mBgColorId);
    }

    // 获取检查列表模式
    public int getCheckListMode() {
        // 返回模式
        return mMode;
    }

    // 获取笔记ID
    public long getNoteId() {
        // 返回笔记ID
        return mNoteId;
    }

    // 获取文件夹ID
    public long getFolderId() {
        // 返回文件夹ID
        return mFolderId;
    }

    // 获取小部件ID
    public int getWidgetId() {
        // 返回小部件ID
        return mWidgetId;
    }

    // 获取小部件类型
    public int getWidgetType() {
        // 返回小部件类型
        return mWidgetType;
    }

    public interface NoteSettingChangedListener {
        /**
         * Called when the background color of current note has just changed
         */
        // 当背景颜色改变时调用
        void onBackgroundColorChanged();

        /**
         * Called when user set clock
         */
        // 当时钟提醒日期改变时调用此方法
        void onClockAlertChanged(long date, boolean set);

        /**
         * Call when user create note from widget
         */
        // 当小部件发生变化时调用
        void onWidgetChanged();

        /**
         * Call when switch between check list mode and normal mode
         * @param oldMode is previous mode before change
         * @param newMode is new mode
         */
        // 当列表模式改变时调用此方法
        void onCheckListModeChanged(int oldMode, int newMode);
    }
}
